import sys

from django.core.management.color import color_style
from django.utils.text import slugify

style = color_style()


class SearchSlug:
    def __init__(self, obj=None, fields=None, sep=None):
        self.warning = None
        self.slug = None
        sep = sep or "|"
        values = []
        if fields:
            for field in fields:
                value = obj
                for f in field.split("."):
                    value = getattr(value, f)
                values.append(value)
        slugs = [slugify(item or "") for item in values]
        slug = f"{sep.join(slugs)}"
        if len(slug) > 250:
            self.warning = f"Warning! Search slug string exceeds 250 chars. See {repr(obj)}\n"
            sys.stdout.write(style.WARNING(self.warning))
        self.slug = slug[:250]



from .search_slug import SearchSlug


class SearchSlugDuplicateFields(Exception):
    pass


class SearchSlugUpdater:
    search_slug_cls = SearchSlug
    sep = "|"

    def __init__(self, fields, model_obj=None):
        if len(fields) > len(list(set(fields))):
            raise SearchSlugDuplicateFields(
                f"Duplicate search slug fields detected. Got {fields}. " f"See {repr(self)}"
            )
        search_slug = self.search_slug_cls(obj=model_obj, fields=fields, sep=self.sep)
        self.warning = search_slug.warning
        self.slug = search_slug.slug



from typing import List

from django.db import models

from .updater import SearchSlugUpdater


class SearchSlugManager(models.Manager):
    search_slug_updater_cls = SearchSlugUpdater
    search_slug_field_name = "slug"

    def update_search_slugs(self) -> None:
        for obj in self.all():
            updater = self.search_slug_updater_cls(
                fields=obj.get_search_slug_fields(), model_obj=obj
            )
            setattr(obj, self.search_slug_field_name, updater.slug)
            obj.save_base(update_fields=[self.search_slug_field_name])


class SearchSlugModelMixin(models.Model):
    search_slug_warning = None
    search_slug_updater_cls = SearchSlugUpdater

    def get_search_slug_fields(self) -> List[str]:
        return []

    slug = models.CharField(
        max_length=250,
        default="",
        null=True,
        editable=False,
        db_index=True,
        help_text="a field used for quick search",
    )

    def save(self, *args, **kwargs):
        updater = self.search_slug_updater_cls(
            fields=self.get_search_slug_fields(), model_obj=self
        )
        self.search_slug_warning = updater.warning
        self.slug = updater.slug
        super().save(*args, **kwargs)

    class Meta:
        abstract = True
